//This code was written by Ensign Justin Shelton, USN.
//Designed to control groups of servos in sequence.
//   6/14/2014

#include <Servo.h>

Servo Servo_RH; //Right hip
Servo Servo_RK; //Right knee
Servo Servo_RA; //Right ankle
Servo Servo_LH; //Left hip
Servo Servo_LK; //Left knee
Servo Servo_LA; //Left ankle

int RHlastposition; //Right hip last position
int RKlastposition; //Right knee last position
int RAlastposition; //Right ankle last position
int LHlastposition; //Left hip last position
int LKlastposition; //Left knee last position
int LAlastposition; //Left ankle last position

bool runOnce=false;

int LA; //new position of the Left ankle
int LK; //new position of the Left knee
int LH; //new position of the Left hip
int RA; //new position of the Right ankle
int RK; //new position of the Right knee
int RH; //new position of the Right hip

void setup()
{
  Servo_LH.attach(4);
  Servo_LK.attach(3);
  Servo_LA.attach(2);
  
  Servo_RH.attach(12);
  Servo_RK.attach(11);
  Servo_RA.attach(10);
}

void detachServos()
{
  Servo_LH.detach(4);
  Servo_LK.detach(3);
  Servo_LA.detach(2);
  
  Servo_RH.detach(12);
  Servo_RK.detach(11);
  Servo_RA.detach(10);
}
  
//This is a function that tells each servo to move to a new positio(an angle)
//int TimeChange indicates how long the servo has to get to it's new position.
//Example:
// ServoTimedMove(45, 45, 45, 45, 45, 45, 200);
// The would make the left ankle move to 45 in 200ms, then the Left knee to 45 in 200ms, and so on.
void ServoTimedMove(int RA, int RK, int RH, int LH, int LK, int LA, int TimeChange)
{
  {
    //Of course, we cannot do a timed move for a servo if we don't know it's position....
    //so this means that for each servo, we need to be give the servo some differences in last
    //position and the new position. We start with the right ankle.
    if ((TimeChange != 0) && (RALastPosition != 1))
    {
      int DeltaRA = RA - RALastPosition; //The total degree change for RA is the new position(RA) minus RAlastposition
      int CycleTime = 20; 
      int CountOfCycles = TimeChange / 20; //this is because 20ms is the natural recycle time for a servo.
      int RADeltaPerCycle = DeltaRA/CountOfCycles;
      
      if (RADeltaPerCycle == 0) //This is for when the changes per cycle is equal to zero
      {
        // If there isn't enough change to do it, then....
        RADeltaPerCycle = (Delta > 0)?1: -1; //this assumes there is a change per cycle.
        CountOfCycles = abs(RADelta); //As you can guess, this takes the absolute value of the change.
        CycleTime = TimeChange/CountOfCycles;
      }
      int RA = RALastPosition;
      
      while (CountOfCycles--)
      {
        RALastPosition += RADeltaPerCycle;
        Servo_RA.write(RA);
        delay(CycleTime);
      }
      Servo_RA.write(RA);
      RALastPosition = RA;
    }
  }
  {
    //This is for the Right knee
    if ((TimeChange != 0) && (RKLastPosition != 1))
    {
      int DeltaRK = RK - RKLastPosition; //The total degree change for RA is the new position(RA) minus RAlastposition
      int CycleTime = 20; 
      int CountOfCycles = TimeChange / 20; //this is because 20ms is the natural recycle time for a servo.
      int RKDeltaPerCycle = DeltaRK/CountOfCycles;
      
      if (RKDeltaPerCycle == 0) //This is for when the changes per cycle is equal to zero
      {
        // If there isn't enough change to do it, then....
        RKDeltaPerCycle = (Delta > 0)?1: -1; //this assumes there is a change per cycle.
        CountOfCycles = abs(RKDelta); //As you can guess, this takes the absolute value of the change.
        CycleTime = TimeChange/CountOfCycles;
      }
      int RK = RKLastPosition;
      
      while (CountOfCycles--)
      {
        RKLastPosition += RKDeltaPerCycle;
        Servo_RK.write(RK);
        delay(CycleTime);
      }
      Servo_RK.write(RK);
      RKLastPosition = RK;
    }
  }
}  

//As you can see, this is a rough idea of my ServoTimedMove function. I'm hoping by now that if you're viewing this
//you may be able to give me word on if im on the right path to getting this program off the ground. I only went through
//the first servos RA and RK instead of doing them all...didn't want to waste time 


//This is a test of the program
void Test()
{
  ServoTimedMove( 90, 90, 90, 90, 90, 90, 300);
  //So, theoretically if all goes well all the motors will whip to 90 degrees in 300ms each.
}
      

void loop()
{
  if (!runOnce)
  {
    Test();
    delay(1000);
    
    detachservos();
    
    runOnce=1; //Set true so the program doesn't run again
  }
}
  
